;;; oni-package.el --- Package manager configuration  -*- lexical-binding: t; -*-

;; Copyright (C) 2019  Tom Willemse

;; Author: Tom Willemse <tom@ryuslash.org>
;; Keywords: local
;; Version: 2023.0613.184755

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:

;; Configuration file for ELPA. This configuration adds the following
;; archives:

;;   - melpa: For all the latest packages.
;;   - org: For the latest version of org-mode.
;;   - oni: For my personal package configuration.

;; This configuration also sets up the proper variables for uploading
;; packages to my personal repository.

;;; Code:

(require 'gnutls)
(require 'package)
(require 'package-x)

;;;; If I let Emacs decide when to run ‘package-initialize’ it seems to do it
;;;; before adding my archives to the ‘package-archives’ variable. By disabling
;;;; the automatic call at startup and telling it to run it only on
;;;; ‘after-init-hook’ it’ll take my added package archives into account and I
;;;; can once again properly see what new packages have been added when I
;;;; refresh the package list.

;;;###autoload
(setq package-enable-at-startup nil)

;;; Disable any package archives to make sure that I can't install any packages
;;; through ELPA. I use Guix and should install packages through there.
;; From: https://github.com/nekifirus/.emacs.d#packages-setup
;;;###autoload
(setq package-archives nil)

(setq package-update-news-on-upload t)

;;; Specify where to upload packages to. Used by
;;; ‘package-upload-buffer’ and ‘package-upload-file’.
(setq package-archive-upload-base
      "bin/")

;;; To fix "Bad Request" errors when downoading from elpa.gnu.org. This turns
;;; off TLS1.3 it seems to me.
(setq gnutls-algorithm-priority "NORMAL:-VERS-TLS1.3")

;;; Temporary fix for `package-maint-add-news-item', until I get my
;;; Copyright assignment back in order.
(defun package-maint-add-news-item (title description archive-url)
  "Add a news item to the webpages associated with the package archive.
TITLE is the title of the news item.
DESCRIPTION is the text of the news item."
  (interactive
   (list
    (read-string "Title: ")
    (read-string "Text: ")
    (let ((archive-name
           (completing-read "Archive: " (mapcar 'car package-archives))))
      (or (alist-get archive-name package-archives nil nil #'string-equal)
          archive-name))))
  (package--update-file "elpa.rss"
			            "<description>"
			            (package--make-rss-entry title description archive-url))
  (package--update-file "news.html"
			            "New entries go here"
			            (package--make-html-entry title description)))

(define-key package-menu-mode-map (kbd "a") 'move-beginning-of-line)
(define-key package-menu-mode-map (kbd "e") 'move-end-of-line)

(add-hook 'package-menu-mode-hook 'hl-line-mode)

;; Set up a keybinding to open a package's URL

(defun oni-package-open-url ()
  (interactive nil package-menu-mode)
  (let* ((pkg-desc (tabulated-list-get-id))
         (url (map-elt (package-desc-extras pkg-desc) :url)))
    (if url
        (browse-url url)
      (error "Package %s doesn't appear to have a url"
             (package-desc-name pkg-desc)))))

(define-key package-menu-mode-map (kbd "o") #'oni-package-open-url)

(provide 'oni-package)
;;; oni-package.el ends here
